@@ -1,24 +1,18 @@
-from GNServer import App as _App, GNRequest, GNResponse, AsyncClient
-from typing import Optional
-import datetime
 import os
 import re
-import signal
-import subprocess
 import sys
 import time
-from typing import Iterable, Set, Tuple
-import os
-from cryptography.hazmat.primitives.ciphers import Cipher, algorithms
-import subprocess
-import sys
-from KeyisBClient import Url
+import signal
 import asyncio
-from io import TextIOWrapper
+import datetime
 import subprocess
+from typing import Set, Tuple, Optional, Iterable, Union
+from GNServer import App as _App, AsyncClient, GNRequest, GNResponse, Url, responses, GNServer as _GNServer
+from KeyisBTools.models.serialization import serialize, deserialize
+from KeyisBTools.cryptography.bytes import userFriendly
 
 def restart_as_root():
-    if os.geteuid() != 0:
+    if os.geteuid() != 0: # type: ignore
         os.execvp("sudo", ["sudo", sys.executable] + sys.argv)
 restart_as_root()
 
@@ -74,7 +68,7 @@ def _kill_process_by_port(port: int):
         if not pids:
             return
         me = os.getpid()
-        for sig in (signal.SIGTERM, signal.SIGKILL):
+        for sig in (signal.SIGTERM, signal.SIGKILL): # type: ignore
             still = set()
             for pid in pids:
                 if pid == me:
@@ -150,7 +144,7 @@ def is_port_in_use(port: int, proto: str = "tcp") -> bool:
 
 class AsyncLogWriter:
     def __init__(self, path: str, flush_interval: int = 3):
-        self.file = open(path, "w+")
+        self.file = open(path, "a+")
         self.flush_interval = flush_interval
         self._stop = False
         self._start = False
@@ -220,6 +214,7 @@ class App():
                            start_when_run: bool = False,
                            venv_path: Optional[str] = None,
                            vm_host: bool = False,
+                           gn_server_crt: Optional[Union[str, bytes]] = None,
                            cert_path: Optional[str] = None,
                            key_path: Optional[str] = None,
                            host: str = '0.0.0.0',
@@ -227,12 +222,12 @@ class App():
                            ):
         
         if log_path is None and self._log_dir_path is not None:
-            log_path = self._log_dir_path + f'/{domain}' + f'{datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")}.log'
+            log_path = self._log_dir_path + f'/{domain}/' + f'{datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")}.log'
 
         if log_path and not os.path.exists(log_path):
             os.makedirs(os.path.dirname(log_path), exist_ok=True)
 
-
+        gn_server_crt = _GNServer._normalize_gn_server_crt(gn_server_crt) if gn_server_crt is not None else None
 
         self._servers_start_files[domain] = {
             "domain": domain,
@@ -244,7 +239,8 @@ class App():
             "cert_path": cert_path if cert_path is not None else self._default_TLS_paths[0] if self._default_TLS_paths is not None else None,
             "key_path": key_path if key_path is not None else self._default_TLS_paths[1] if self._default_TLS_paths is not None else None,
             "host": host,
-            "log_path": log_path
+            "log_path": log_path,
+            "gn_server_crt_data_raw_bytes": gn_server_crt,
             }
 
         
@@ -260,126 +256,116 @@ class App():
             if self._servers_start_files[server]["start_when_run"]:
                 asyncio.create_task(self.startServer(server))
 
+
     async def startServer(self, domain: str, timeout: float = 30):
-        if domain in self._servers_start_files:
-
-
-            res = await self.checkServerHealth(domain, timeout=1.0)
-            if res[0]:
-                raise ValueError(f"Server alredy running: {domain}")
-            
-
-
-            server = self._servers_start_files[domain]
-            path = server["path"]
-            port = server["port"]
-            venv_path = server["venv_path"]
-            vm_host = server['vm_host']
-            host = server['host']
-            
-            cert_path = server['cert_path']
-            key_path = server['key_path']
-            
-            if vm_host and (cert_path is None or key_path is None):
-                raise Exception('Не указаны сертификаты')
-            
-            if not os.path.isfile(path):
-                raise ValueError(f"Server start file not found: {path}")
-            
-            
-            if is_port_in_use(port, 'udp'):
-                _kill_process_by_port(port)
-                await asyncio.sleep(1)
-
-
-            out_ = None
-
-            if path.endswith('.py'):
-                if venv_path is not None:
-                    if not os.path.isdir(venv_path):
-                        raise ValueError(f"Virtual environment path not found: {venv_path}")
-                    python_executable = os.path.join(venv_path, 'bin', 'python')
-                    if not os.path.isfile(python_executable):
-                        raise ValueError(f"Python executable not found in virtual environment: {python_executable}")
-                else:
-                    python_executable = sys.executable
-
-                if not vm_host:
-                    argv = []
-                else:
-                    argv = [
-                        'gn:vm-host:start',
-                        str(domain),
-                        str(port),
-                        str(cert_path),
-                        str(key_path),
-                        str(host)
-                    ]
-
-                self.writeLog(domain, 'Server starting...')
-                proc = subprocess.Popen(
-                    [python_executable, path, *argv],
-                    stdout=subprocess.PIPE,
-                    stderr=subprocess.PIPE,
-                    text=True
-                )
-
-                save_to_out = True
-                save_out = []
-
-                async def log_stream(stream, prefix):
-                    while True:
-                        line = await asyncio.to_thread(stream.readline)
-                        if not line:
-                            break
-
-                        if save_to_out:
-                            save_out.append(line.strip())
-
-                        self.writeLog(domain, f"{prefix}: {line.strip()}")
-
-                asyncio.create_task(log_stream(proc.stdout, ""))
-                asyncio.create_task(log_stream(proc.stderr, "ERR"))
-
-                try:
-                    returncode = await asyncio.to_thread(proc.wait, 3)
-                    self.writeLog(domain, f"Process exited with code {returncode}")
-                    save_out.clear()
-                except subprocess.TimeoutExpired:
-                    self.writeLog(domain, "Process timed out")
-                    out_ = '\n'.join(save_out)
-
-                save_to_out = False
-
-                # try:
-                #     returncode = proc.wait(timeout=3)
-                #     out, err = proc.communicate()
-                #     self.writeLog(domain, f'Server error: {returncode}')
-                #     self.writeLog(domain, f'stdout: {out}')
-                #     self.writeLog(domain, f'stderr: {err}')
-
-                #     print("Процесс завершился с кодом:", returncode)
-                #     print("STDOUT:", out)
-                #     print("STDERR:", err)
-                #     out_ = out + '\n\n' + err
-                # except subprocess.TimeoutExpired:
-                #     pass
-                
-
-            else:
-                self.writeLog(domain, 'Server starting...')
-                subprocess.Popen([path], stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
-            
-            r =  await self.checkServerHealth(domain, timeout=timeout, out=out_)
-            self.writeLog(domain, f'Server checkServerHealth status-> {r[0]}')
-            return r
-        else:
+        if domain not in self._servers_start_files:
             self.writeLog(domain, f'No server start file found with domain: {domain}')
             raise ValueError(f"No server start file found with domain: {domain}")
-    
+
+        # Проверка что уже запущен
+        res = await self.checkServerHealth(domain, timeout=1.0)
+        if res[0]:
+            return (f"Server already running: {domain}")
+
+        server = self._servers_start_files[domain]
+        path = server["path"]
+        port = server["port"]
+        venv_path = server["venv_path"]
+        vm_host = server['vm_host']
+        host = server['host']
+
+        cert_path = server['cert_path']
+        key_path = server['key_path']
+
+        if vm_host and (cert_path is None or key_path is None):
+            raise Exception('Не указаны сертификаты')
+
+        if not os.path.isfile(path):
+            raise ValueError(f"Server start file not found: {path}")
+
+        if is_port_in_use(port, 'udp'):
+            _kill_process_by_port(port)
+            await asyncio.sleep(1)
+
+        out_ = None
+
+        if path.endswith('.py'):
+            # выбираем python
+            if venv_path is not None:
+                if not os.path.isdir(venv_path):
+                    raise ValueError(f"Virtual environment path not found: {venv_path}")
+                python_executable = os.path.join(venv_path, 'bin', 'python')
+                if not os.path.isfile(python_executable):
+                    raise ValueError(f"Python executable not found in virtual environment: {python_executable}")
+            else:
+                python_executable = sys.executable
+
+            if not vm_host:
+                argv = {}
+            else:
+                argv = {
+                    'command': 'gn:vm-host:start',
+                    'domain': str(domain),
+                    'port': str(port),
+                    'host': str(host)
+                }
+            
+            if cert_path is not None:
+                argv['cert_path'] = cert_path
+
+            if key_path is not None:
+                argv['key_path'] = key_path
+
+            
+            if server['gn_server_crt_data_raw_bytes'] is not None:
+                argv['gn_server_crt'] = server['gn_server_crt_data_raw_bytes']
+
+            raw_argv = userFriendly.encode(serialize(argv))
+
+            self.writeLog(domain, 'Server starting...')
+
+            # асинхронный запуск процесса
+            proc = await asyncio.create_subprocess_exec(
+                python_executable, path, raw_argv,
+                stdout=asyncio.subprocess.PIPE,
+                stderr=asyncio.subprocess.PIPE
+            )
+
+            save_out = []
+
+            async def log_stream(stream: asyncio.StreamReader, prefix: str):
+                async for line in stream:
+                    text = line.decode().rstrip()
+                    save_out.append(text)
+                    self.writeLog(domain, f"{prefix}: {text}")
+
+            # запускаем таски на чтение stdout/stderr
+            asyncio.create_task(log_stream(proc.stdout, "OUT"))
+            asyncio.create_task(log_stream(proc.stderr, "ERR"))
+
+            try:
+                # ждём чуть-чуть: если процесс сразу сдохнет — фиксируем
+                returncode = await asyncio.wait_for(proc.wait(), timeout=3)
+                self.writeLog(domain, f"Process exited with code {returncode}")
+                save_out.clear()
+            except asyncio.TimeoutError:
+                self.writeLog(domain, "Process timed out")
+                out_ = '\n'.join(save_out)
+
+        else:
+            self.writeLog(domain, 'Server starting...')
+            await asyncio.create_subprocess_exec(
+                path,
+                stdout=asyncio.subprocess.DEVNULL,
+                stderr=asyncio.subprocess.DEVNULL
+            )
+
+        r = await self.checkServerHealth(domain, timeout=timeout, out=out_)
+        self.writeLog(domain, f'Server checkServerHealth status-> {r[0]}')
+        return r
 
 
-    async def _send_message_to_server(self, domain: str, path: str, payload: Optional[dict] = None, timeout: float = 1.0, res: list = None) -> Optional[GNResponse]:
+    async def _send_message_to_server(self, domain: str, path: str, payload: Optional[dict] = None, timeout: float = 1.0, res: list = []) -> Optional[GNResponse]:
         port = self._servers_start_files[domain].get("port")
         if port is None:
             raise ValueError(f"No port specified for server: {domain}")
@@ -455,8 +441,8 @@ class App():
         self._app.run(
             domain=domain,
             port=port,
-            cert_path=cert_path,
-            key_path=key_path,
+            tls_certfile=cert_path,
+            tls_keyfile=key_path,
 
             host=host,
             idle_timeout=idle_timeout,
@@ -488,7 +474,11 @@ class App():
                 try:
                     result = await self.checkServerHealth(domain, timeout=timeout)
                     if result[0]:
-                        return GNResponse('ok', {'message': f'Server {domain} is alive.', 'time': result[2].payload['time']})
+                        try:
+                            _time = result[2].payload.get('time')
+                        except:
+                            _time = None
+                        return GNResponse('ok', {'message': f'Server {domain} is alive.', 'time': _time})
                     else:
                         return GNResponse('error', {'error': f'Server {domain} is not responding.'})
                 except ValueError as e:
